/*

  System (iOS) V.1.0.0.
  XII - 2012 Javi VR
  www.neptalium.com


  Example:

    if application:getDeviceInfo() == "iOS" then
      require "system"

      -- Misc functions
      print("Name: " ..                          system.name())
      print("Model: " ..                         system.model())
      print("System name: " ..                   system.systemName())
      print("System version: " ..                system.systemVersion())
      print("Has retina display: " ..            tostring(system.hasRetinaDisplay()))
      print("Has FPU (Floating Point Unit): " .. tostring(system.hasFPU()))
      print("Is multitasking supported: " ..     tostring(system.isMultitaskingSupported()))
      print("CPU byte order (1234 or 4321): " .. system.byteOrder())
      print("Number of CPUs: " ..                system.numCPUs())
      print("CPU frequency (Hz): " ..            system.cpuFrequency())
      print("Bus frequency (Hz): " ..            system.busFrequency())
      print("Physical memory (bytes): " ..       system.physicalMemory())
      print("User memory (bytes): " ..           system.userMemory())
      print("Free memory (bytes): " ..           system.freeMemory())
      print("Total disk space (bytes): " ..      system.totalDiskSpace())
      print("Free disk space (bytes): " ..       system.freeDiskSpace())

      -- Battery state and level
      print("Battery monitoring enabled: " .. tostring(system.isBatteryMonitoringEnabled()))
      system.enableBatteryMonitoring()
      print("Battery state (unplugged, charging, full or unknown): " .. system.batteryState())
      print("Battery level (0 - 1): " .. system.batteryLevel())
      system.disableBatteryMonitoring()

      -- Proximity sensor
      system.enableProximityMonitoring()
      if system.isProximityMonitoringEnabled() then
        print("Proximity sensor state (on, off): " .. tostring(system.proximityState()))
      else
        print("Proximity sensor not supported!")
      end
      system.disableProximityMonitoring()
    end
*/


#import "gideros.h"
#import "UIKit/UIDevice.h"
#import "sys/sysctl.h"
#import "mach/mach.h"


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int getSysInfo(uint type)
{
  int    result;
  int    mib[2] = { CTL_HW, type };
  size_t size   = sizeof(int);
  sysctl(mib, 2, &result, &size, NULL, 0);

  return result;
}


// Available in iOS 2.0
// ----------------------------------------------------------------------------
static int name(lua_State *L)
{
  lua_pushstring(L, [[[UIDevice currentDevice] name] UTF8String]);

  return 1;
}


// Available in iOS 2.0
// ----------------------------------------------------------------------------
static int model(lua_State *L)
{
  lua_pushstring(L, [[[UIDevice currentDevice] model] UTF8String]);

  return 1;
}


// Available in iOS 2.0
// ----------------------------------------------------------------------------
static int systemName(lua_State *L)
{
  lua_pushstring(L, [[[UIDevice currentDevice] systemName] UTF8String]);

  return 1;
}


// Available in iOS 2.0
// ----------------------------------------------------------------------------
static int systemVersion(lua_State *L)
{
  lua_pushstring(L, [[[UIDevice currentDevice] systemVersion] UTF8String]);

  return 1;
}


// Available in iOS 4.0
// ----------------------------------------------------------------------------
static int hasRetinaDisplay(lua_State *L)
{
  lua_pushboolean(L, [UIScreen mainScreen].scale == 2.0f);

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int hasFPU(lua_State *L)
{
  lua_pushboolean(L, getSysInfo(HW_FLOATINGPT));

  return 1;
}


// Available in iOS 4.0
// ----------------------------------------------------------------------------
static int isMultitaskingSupported(lua_State *L)
{
  lua_pushboolean(L, [[UIDevice currentDevice] isMultitaskingSupported]);

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int byteOrder(lua_State *L)
{
  lua_pushnumber(L, getSysInfo(HW_BYTEORDER));

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int numCPUs(lua_State *L)
{
  lua_pushnumber(L, getSysInfo(HW_NCPU));

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int cpuFrequency(lua_State *L)
{
  lua_pushnumber(L, getSysInfo(HW_CPU_FREQ));

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int busFrequency(lua_State *L)
{
  lua_pushnumber(L, getSysInfo(HW_BUS_FREQ));

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int physicalMemory(lua_State *L)
{
  lua_pushnumber(L, getSysInfo(HW_PHYSMEM));

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int userMemory(lua_State *L)
{
  lua_pushnumber(L, getSysInfo(HW_USERMEM));

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int freeMemory(lua_State *L)
{
  mach_port_t            host_port = mach_host_self();
  mach_msg_type_number_t host_size = sizeof(vm_statistics_data_t) / sizeof(integer_t);
  vm_size_t              page_size;
  vm_statistics_data_t   vm_stat;
  host_page_size(host_port, &page_size);
  host_statistics(host_port, HOST_VM_INFO, (host_info_t)&vm_stat, &host_size);
  double result = vm_stat.free_count * page_size;

  lua_pushnumber(L, result);

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int totalDiskSpace(lua_State *L)
{
  double        result     = 0.0;
  NSDictionary *dictionary = [[NSFileManager defaultManager] attributesOfFileSystemForPath:NSHomeDirectory() error:nil];
  if(dictionary)
  {
    NSNumber *size = [dictionary objectForKey: NSFileSystemSize];
    result         = [size doubleValue];
  }

  lua_pushnumber(L, result);

  return 1;
}


// Available in iOS 1.0 ?
// ----------------------------------------------------------------------------
static int freeDiskSpace(lua_State *L)
{
  double        result     = 0.0;
  NSDictionary *dictionary = [[NSFileManager defaultManager] attributesOfFileSystemForPath:NSHomeDirectory() error:nil];
  if(dictionary)
  {
    NSNumber *size = [dictionary objectForKey:NSFileSystemFreeSize];
    result         = [size doubleValue];
  }

  lua_pushnumber(L, result);

  return 1;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int isBatteryMonitoringEnabled(lua_State *L)
{
  lua_pushboolean(L, [[UIDevice currentDevice] isBatteryMonitoringEnabled]);

  return 1;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int enableBatteryMonitoring(lua_State *L)
{
  [[UIDevice currentDevice] setBatteryMonitoringEnabled:YES];

  return 0;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int disableBatteryMonitoring(lua_State *L)
{
  [[UIDevice currentDevice] setBatteryMonitoringEnabled:NO];

  return 0;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int batteryLevel(lua_State *L)
{
  double               level = -1.0;
  UIDeviceBatteryState state = [[UIDevice currentDevice] batteryState];
  if(state != UIDeviceBatteryStateUnknown)
  {
    level = [[UIDevice currentDevice] batteryLevel];
  }

  lua_pushnumber(L, level);

  return 1;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int batteryState(lua_State *L)
{
  const char          *result = NULL;
  UIDeviceBatteryState state  = [[UIDevice currentDevice] batteryState];
  switch(state)
  {
    case UIDeviceBatteryStateUnplugged:
      result = "unplugged";
      break;
    case UIDeviceBatteryStateCharging:
      result = "charging";
      break;
    case UIDeviceBatteryStateFull:
      result = "full";
      break;
    case UIDeviceBatteryStateUnknown:
    default:
      result = "unknown";
  }

  lua_pushstring(L, result);

  return 1;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int isProximityMonitoringEnabled(lua_State *L)
{
  lua_pushboolean(L, [[UIDevice currentDevice] isProximityMonitoringEnabled]);

  return 1;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int enableProximityMonitoring(lua_State *L)
{
  [[UIDevice currentDevice] setProximityMonitoringEnabled:YES];

  return 0;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int disableProximityMonitoring(lua_State *L)
{
  [[UIDevice currentDevice] setProximityMonitoringEnabled:NO];

  return 0;
}


// Available in iOS 3.0
// ----------------------------------------------------------------------------
static int proximityState(lua_State *L)
{
  lua_pushboolean(L, [[UIDevice currentDevice] proximityState]);

  return 1;
}


// ----------------------------------------------------------------------------
static int loader(lua_State *L)
{
  const luaL_Reg func_list[] =
  {
    { "name",                         name },
    { "model",                        model },
    { "systemName",                   systemName },
    { "systemVersion",                systemVersion },
    { "hasRetinaDisplay",             hasRetinaDisplay },
    { "hasFPU",                       hasFPU },
    { "isMultitaskingSupported",      isMultitaskingSupported },
    { "byteOrder",                    byteOrder },
    { "numCPUs",                      numCPUs },
    { "cpuFrequency",                 cpuFrequency },
    { "busFrequency",                 busFrequency },
    { "physicalMemory",               physicalMemory },
    { "userMemory",                   userMemory },
    { "freeMemory",                   freeMemory },
    { "totalDiskSpace",               totalDiskSpace },
    { "freeDiskSpace",                freeDiskSpace },
    { "isBatteryMonitoringEnabled",   isBatteryMonitoringEnabled },
    { "enableBatteryMonitoring",      enableBatteryMonitoring },
    { "disableBatteryMonitoring",     disableBatteryMonitoring },
    { "batteryLevel",                 batteryLevel },
    { "batteryState",                 batteryState },
    { "isProximityMonitoringEnabled", isProximityMonitoringEnabled },
    { "enableProximityMonitoring",    enableProximityMonitoring },
    { "disableProximityMonitoring",   disableProximityMonitoring },
    { "proximityState",               proximityState },
    { NULL, NULL }
  };

  luaL_register(L, "System", func_list);

  return 1;
}


// ----------------------------------------------------------------------------
static void g_initializePlugin(lua_State *L)
{
  lua_getglobal(L, "package");
  lua_getfield(L, -1, "preload");
  lua_pushcfunction(L, loader);
  lua_setfield(L, -2, "system");
  lua_pop(L, 2);
}


// ----------------------------------------------------------------------------
static void g_deinitializePlugin(lua_State *L)
{

}


REGISTER_PLUGIN("system", "1.0")
